/*  VirtualCue XMMS
 *  Copyright (C) 2005 Barraud Manuel
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef CUE_PARSER_H
#define CUE_PARSER_H

#define SIZE_IT_PERFORMER 255
#define SIZE_IT_TITLE 255
#define SIZE_IT_FILE 1024

typedef struct _InfoTrack
{
      char Performer[SIZE_IT_PERFORMER];
      char Title[SIZE_IT_TITLE];
      long time_millieme;
}InfoTrack;

typedef struct _InfoCue
{
char Performer[SIZE_IT_PERFORMER];
char Title[SIZE_IT_TITLE];
char File[SIZE_IT_FILE];
int nbtrack;
InfoTrack tracks[1];
}InfoCue;


int CalcNbTrackAvailable(char *filename);
InfoCue *GetInfoCue(char *filename);
long GetTimeByNum(InfoCue *Icue,int num,int totaltime);
#endif
